
ApplicationCoordinator subclass: #RegionEditor
  instanceVariableNames: 
    ' drawingPane '
  classVariableNames: ''
  poolDictionaries: ''
  categories: #( 'RegionEditor' )

!

Object subclass: #RegionEditorObject
  instanceVariableNames: 
    ' position '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants '
  categories: #( 'RegionEditor' )

!

RegionEditorObject subclass: #RegionEditorArea
  instanceVariableNames: 
    ' vertices lines '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants '
  categories: #( 'RegionEditor' )

!

RegionEditorObject subclass: #RegionEditorLine
  instanceVariableNames: 
    ' vertices areas '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants '
  categories: #( 'RegionEditor' )

!

RegionEditorObject subclass: #RegionEditorVertex
  instanceVariableNames: 
    ' lines areas '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants '
  categories: #( 'RegionEditor' )

!

SubPane subclass: #ScrollingPane
  instanceVariableNames: 
    ' scrollRectangle scrollPosition mouseLocation lastModifierKeys '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants VirtualKeyConstants '
  categories: #( 'RegionEditor' )

!

ScrollingPane subclass: #DrawingPane
  instanceVariableNames: 
    ' objects selectedObjects buffer '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants VirtualKeyConstants '
  categories: #( 'RegionEditor' )

!


!CodeFiler forClass: RegionEditor class !
   
buildView: aView forModel: anApplication
    "Create the structure of the window."

    ^anApplication buildView: aView

! !

(CodeFiler organizerFor: RegionEditor class) reorganizeFrom: #(
('building' buildView:forModel: )) !


!CodeFiler forClass: RegionEditor !
 
objects: objects movedBy: aPoint
    "The selected objects have been moved by aPoint. Change their position."
    | vertices |
    vertices := objects inject: Set new into: [:set :object |
        set addAll: object lowestComponents; yourself].
    vertices do: [:vertex |
        vertex position: vertex position + aPoint]

!
  
objects: objects near: aPoint
    " Return the most primitive type of object first."
    | list |
    list := objects select: [:object | object isVertex and: [object isNearPoint: aPoint]].
    list isEmpty ifFalse: [^list].
    list := objects select: [:object | object isLine and: [object isNearPoint: aPoint]].
    list isEmpty ifFalse: [^list].
    ^objects select: [:object | object isArea and: [object isNearPoint: aPoint]].

!
   
objectMenu: aPane
    "Set the menu of the drawingPane"
    | menu |
    menu := Menu
        labels: '&Add Region\&Delete Region\&Inspect' withCrs
        actions: #(addRegion deleteRegion inspect).
    menu owner: self.
    aPane setMenu: menu
!
 
objectsDeleted: objects
    "We will only handle deletion of areas."
    | areas components deleteSet addSet |
    areas := objects select: [:object | object isArea].
    components := areas inject: Set new into: [:set :object |
        set addAll: object components; yourself].
    deleteSet := components select: [:object |
        drawingPane objects includes: object].
    addSet := objects reject: [:object |
        components includes: object].
    drawingPane deleteObjects: deleteSet refresh: false.
    drawingPane addObjects: addSet refresh: false
!

objectsDisplaySelected: selected unselected: unselected
    "Display the objects."
    unselected do: [:object | object displayWith: drawingPane pen].
    selected do: [:object | object displaySelectedWith: drawingPane pen].



!

objectControlClickIn: aPane
    "Insert new vertex in the middle of the line that is nearby."
    | aPoint areas startVertex endVertex newVertex newLine |
    aPoint := aPane mouseLocation.
    aPane objects do: [:object |
        (object isLine and: [object isNearPoint: aPoint]) ifTrue: [
            areas := object areas.
            startVertex := object vertices first.
            endVertex := object vertices last.
            newVertex := RegionEditorVertex from: aPoint.
            newLine := RegionEditorLine from: newVertex to: endVertex.
            object replace: endVertex with: newVertex.
            areas do: [:eachArea |
                eachArea add: newVertex between: startVertex and: endVertex.
                eachArea addLine: newLine].
            aPane
                addObject: newVertex refresh: false;
                addObject: newLine refresh: false;
                selectObject: newVertex.
            ^aPane moveFrom: aPoint]]
! !

(CodeFiler organizerFor: RegionEditor) reorganizeFrom: #(
('event handlers' objects:movedBy: objects:near: objectMenu: objectsDeleted: objectsDisplaySelected:unselected: objectControlClickIn: )) !

!CodeFiler forClass: RegionEditor !
 
addRegion
    "Add a region to the drawing."
    | aPoint vertexPoints area newObjects |
    aPoint := drawingPane mouseLocation.
    vertexPoints := OrderedCollection
        with: (aPoint leftAndUp: 10 @ 10)
        with: (aPoint rightAndUp: 10 @ 10)
        with: (aPoint rightAndDown: 10 @ 10)
        with: (aPoint leftAndDown: 10 @ 10).
    area := RegionEditorArea from: vertexPoints.
    newObjects := area components.
    newObjects do: [:object |
        drawingPane addObject: object refresh: false].
    drawingPane selectObject: area
!
  
deleteRegion
    "Delete the selected region from the drawing."
    drawingPane deleteSelectedObjects.
! !

(CodeFiler organizerFor: RegionEditor) reorganizeFrom: #(
('menu handlers' addRegion deleteRegion )) !

!CodeFiler forClass: RegionEditor !
 
buildView: aView
    "Build the view of the application."
    aView
        label: 'Region Editor';
        noSmalltalkMenuBar;
        addSubpane: ((drawingPane := DrawingPane new)
            owner: self;
            when: #controlClickAt: send: #objectControlClickIn: to: self with: drawingPane;
            when: #objects:movedBy: send: #objects:movedBy:  to: self;
            when: #objectsDeleted: send: #objectsDeleted: to: self;
            when: #objects:near: send: #objects:near:  to: self;
            when: #objectsDisplaySelected:unselected: send: #objectsDisplaySelected:unselected: to: self;
            when: #needsMenu: send: #objectMenu: to: self with: drawingPane).
    drawingPane foreColor: Color black
! !

(CodeFiler organizerFor: RegionEditor) reorganizeFrom: #(
('private-building' buildView: )) !
CodeFiler setCommentFor: RegionEditor to: ''!


!CodeFiler forClass: RegionEditorObject class ! !

(CodeFiler organizerFor: RegionEditorObject class) reorganizeFrom: #() !

!CodeFiler forClass: RegionEditorObject class !
  
new
    ^super new initialize
! !

(CodeFiler organizerFor: RegionEditorObject class) reorganizeFrom: #(
('instance creation' new )) !


!CodeFiler forClass: RegionEditorObject !
  
initialize
    "Do nothing"
! !

(CodeFiler organizerFor: RegionEditorObject) reorganizeFrom: #(
('initializing' initialize )) !

!CodeFiler forClass: RegionEditorObject !
  
position
    ^position
!
  
recalculatePosition
    "Do nothing"
!

boundingBox
    ^position extentFromLeftTop: self extent
!

extent
    ^self implementedBySubclass
! !

(CodeFiler organizerFor: RegionEditorObject) reorganizeFrom: #(
('accessing' position recalculatePosition boundingBox extent )) !

!CodeFiler forClass: RegionEditorObject !
 
isArea
    ^false
!
   
isNearPoint: aPoint
    ^self boundingBox containsPoint: aPoint
!
 
isVertex
    ^false
!
 
isLine
    ^false
! !

(CodeFiler organizerFor: RegionEditorObject) reorganizeFrom: #(
('testing' isArea isNearPoint: isVertex isLine )) !

!CodeFiler forClass: RegionEditorObject !

displaySelectedWith: aPen
    | oldColor |
    oldColor := aPen foreColor.
    aPen foreColor: Color darkRed.
    self displayWith: aPen.
    aPen foreColor: oldColor

! !

(CodeFiler organizerFor: RegionEditorObject) reorganizeFrom: #(
('displaying' displaySelectedWith: )) !

!CodeFiler forClass: RegionEditorObject !
 
printOn: aStream
    aStream
        nextPutAll: self printOnDescription;
        nextPutAll: ' ('.
    self printInternalOn: aStream.
    aStream nextPut: $)
!
  
printOnDescription
    ^self class name
! !

(CodeFiler organizerFor: RegionEditorObject) reorganizeFrom: #(
('printing' printOn: printOnDescription )) !
CodeFiler setCommentFor: RegionEditorObject to: ''!


!CodeFiler forClass: RegionEditorArea class !
  
from: aCollection

    ^self new from: aCollection! !

(CodeFiler organizerFor: RegionEditorArea class) reorganizeFrom: #(
('instance creation' from: )) !


!CodeFiler forClass: RegionEditorArea !

vertexPoints
    ^vertices collect: [:each | each position]
!
 
vertices
    ^vertices
!
  
add: aVertex between: startVertex and: endVertex
    | startIndex endIndex |
    aVertex addArea: self.
    startIndex := vertices indexOf: startVertex.
    endIndex := vertices indexOf: endVertex.
    startIndex < endIndex | (endIndex = 1)
        ifTrue: [vertices add: aVertex afterIndex: startIndex]
        ifFalse: [vertices add: aVertex afterIndex: endIndex]
!
 
recalculatePosition
    (vertices isNil or: [vertices isEmpty])
        ifTrue: [^self].
    position := self vertexPoints
        inject: vertices first position
        into: [:sum :each |
            sum leftMostAndHighest: each]
!
   
replace: aVertex with: anotherVertex
    | index |
    index := vertices indexOf: aVertex.
    vertices at: index put: anotherVertex.
    anotherVertex addArea: self.
    self recalculatePosition
!
 
addLine: aLine
    (self includesLine: aLine) ifFalse: [
        lines add: aLine.
        aLine addArea: self]
!
   
lowestComponents
    ^vertices
!
  
extent
    ^(self vertexPoints
        inject: vertices first position
        into: [:sum :each |
            sum rightMostAndLowest: each]) - position
!
 
lines
    ^lines
!

components
    ^(WriteStream on: (Array new: 20))
        nextPut: self;
        nextPutAll: lines;
        nextPutAll: vertices;
        contents
!
  
removeLine: aLine
    lines remove: aLine ifAbsent: []
! !

(CodeFiler organizerFor: RegionEditorArea) reorganizeFrom: #(
('accessing' vertexPoints vertices add:between:and: recalculatePosition replace:with: addLine: lowestComponents extent lines components removeLine: )) !

!CodeFiler forClass: RegionEditorArea !
  
isArea
    ^true
!

includesLine: aLine
    lines do: [:each |
        (each isSameAs: aLine)
            ifTrue: [^true]].
    ^false
! !

(CodeFiler organizerFor: RegionEditorArea) reorganizeFrom: #(
('testing' isArea includesLine: )) !

!CodeFiler forClass: RegionEditorArea !
   
displaySelectedWith: aPen
    aPen
        setLineWidth: 1;
        setHatchBrush: HsFdiagonal color: aPen foreColor.
    aPen polygonFilled: self vertexPoints.
    aPen
        setBrushStyle: BsSolid color: aPen foreColor hatch: nil
!
  
displayWith: aPen
    aPen
        setLineWidth: 1;
        polygon: self vertexPoints.
! !

(CodeFiler organizerFor: RegionEditorArea) reorganizeFrom: #(
('displaying' displaySelectedWith: displayWith: )) !

!CodeFiler forClass: RegionEditorArea !
   
printInternalOn: aStream
    aStream print: position
! !

(CodeFiler organizerFor: RegionEditorArea) reorganizeFrom: #(
('printing' printInternalOn: )) !

!CodeFiler forClass: RegionEditorArea !
   
from: aCollection
    vertices := aCollection collect: [:each | RegionEditorVertex from: each].
    lines := RegionEditorLine linesFromVertices: vertices.
    vertices do: [:vertex | vertex addArea: self].
    lines do: [:line |
        line vertices do: [:vertex | vertex addLine: line].
        line
            addArea: self;
            recalculatePosition].
    self recalculatePosition
! !

(CodeFiler organizerFor: RegionEditorArea) reorganizeFrom: #(
('private' from: )) !
CodeFiler setCommentFor: RegionEditorArea to: ''!


!CodeFiler forClass: RegionEditorLine class !
 
from: oneRegionEditorVertex to: anotherRegionEditorVertex
    ^self new from: oneRegionEditorVertex to: anotherRegionEditorVertex
!
   
linesFromVertices: aCollection
    | stream lines |
    stream := ReadStream on: aCollection, (Array with: aCollection first).
    lines := OrderedCollection new: aCollection size + 1.
    aCollection size timesRepeat: [
        lines add: (self from: stream next to: stream peek)].
    ^lines
! !

(CodeFiler organizerFor: RegionEditorLine class) reorganizeFrom: #(
('instance creation' from:to: linesFromVertices: )) !


!CodeFiler forClass: RegionEditorLine !
 
initialize
    super initialize.
    areas := OrderedCollection new
! !

(CodeFiler organizerFor: RegionEditorLine) reorganizeFrom: #(
('initializing' initialize )) !

!CodeFiler forClass: RegionEditorLine !
 
vertices
    ^vertices
!
  
addArea: anArea
    (areas includes: anArea)
        ifFalse: [areas add: anArea]
!
  
recalculatePosition
    position := vertices first position leftMostAndHighest: vertices last position
!
  
replace: aVertex with: anotherVertex
    | index |
    index := vertices indexOf: aVertex.
    vertices at: index put: anotherVertex.
    anotherVertex addLine: self.
    self recalculatePosition
!
 
areas
    ^areas
!

lowestComponents
    ^vertices
!
  
extent
    ^(vertices first position rightMostAndLowest: vertices last position) - position
! !

(CodeFiler organizerFor: RegionEditorLine) reorganizeFrom: #(
('accessing' vertices addArea: recalculatePosition replace:with: areas lowestComponents extent )) !

!CodeFiler forClass: RegionEditorLine !
  
isSameAs: aRegionEditorLine
    ^vertices = aRegionEditorLine vertices or: [vertices = aRegionEditorLine vertices reversed]
! !

(CodeFiler organizerFor: RegionEditorLine) reorganizeFrom: #(
('comparing' isSameAs: )) !

!CodeFiler forClass: RegionEditorLine !
  
isNearPoint: aPoint
    ^self
        doesPoint: aPoint
        collideWithLineFrom: vertices first position
        to: vertices last position
        within: 4

!
 
isLine
    ^true
! !

(CodeFiler organizerFor: RegionEditorLine) reorganizeFrom: #(
('testing' isNearPoint: isLine )) !

!CodeFiler forClass: RegionEditorLine !
 
displaySelectedWith: aPen
    aPen
        setLineWidth: 3;
        place: vertices first position;
        goto: vertices last position
!
 
displayWith: aPen
    aPen
        setLineWidth: 1;
        place: vertices first position;
        goto: vertices last position

! !

(CodeFiler organizerFor: RegionEditorLine) reorganizeFrom: #(
('displaying' displaySelectedWith: displayWith: )) !

!CodeFiler forClass: RegionEditorLine !
   
printInternalOn: aStream
    aStream
        print: vertices first;
        nextPutAll: ' -> ';
        print: vertices last
! !

(CodeFiler organizerFor: RegionEditorLine) reorganizeFrom: #(
('printing' printInternalOn: )) !

!CodeFiler forClass: RegionEditorLine !

from: oneVertex to: anotherVertex
    vertices := Array with: oneVertex with: anotherVertex.
    oneVertex addLine: self.
    anotherVertex addLine: self.
    self recalculatePosition
!
  
doesPoint: p collideWithArbitraryLineFrom: p0 to: p1 within: threshold
    "i represents the projection point of p on the line from p0 to p1"
    | v i |
    v := p1 - p0.
    i := p0 + (v * ((p - p0) dotProduct: v) / (v dotProduct: v)).
    ^((i x between: (p0 x min: p1 x) and: (p0 x max: p1 x)) and:
        [i y between: (p0 y min: p1 y) and: (p0 y max: p1 y)]) and:
        [((p x - i x) squared + (p y - i y) squared) <= threshold squared]

!
   
doesPoint: aPoint collideWithLineFrom: startPoint to: endPoint within: threshold
    "If the points coincide, check for the distance between the points only."
    ^startPoint = endPoint
        ifTrue: [((aPoint x - startPoint x) squared + (aPoint y - startPoint y) squared) <= threshold squared]
        ifFalse: [self doesPoint: aPoint collideWithArbitraryLineFrom: startPoint to: endPoint within: threshold]

! !

(CodeFiler organizerFor: RegionEditorLine) reorganizeFrom: #(
('private' from:to: doesPoint:collideWithArbitraryLineFrom:to:within: doesPoint:collideWithLineFrom:to:within: )) !
CodeFiler setCommentFor: RegionEditorLine to: ''!


!CodeFiler forClass: RegionEditorVertex class !
 
from: aPoint
    ^self new from: aPoint
! !

(CodeFiler organizerFor: RegionEditorVertex class) reorganizeFrom: #(
('instance creation' from: )) !


!CodeFiler forClass: RegionEditorVertex !
  
initialize
    super initialize.
    lines := OrderedCollection new.
    areas := OrderedCollection new
! !

(CodeFiler organizerFor: RegionEditorVertex) reorganizeFrom: #(
('initializing' initialize )) !

!CodeFiler forClass: RegionEditorVertex !

position: aPoint
    position := aPoint.
    lines do: [:each |
        each recalculatePosition].
    areas do: [:each |
        each recalculatePosition].
!

addArea: anArea
    (areas includes: anArea)
        ifFalse: [areas add: anArea]
!
  
boundingBox
    ^super boundingBox expandBy: 2
!
  
areas
    ^areas
!

addLine: aLine
    (self includesLine: aLine)
        ifFalse: [lines add: aLine]
!
  
lowestComponents
    ^Array with: self
!
  
extent
    ^1@1
!
 
lines
   ^lines
!
 
anyArea
    ^areas first
! !

(CodeFiler organizerFor: RegionEditorVertex) reorganizeFrom: #(
('accessing' position: addArea: boundingBox areas addLine: lowestComponents extent lines anyArea )) !

!CodeFiler forClass: RegionEditorVertex !
   
includesLine: aLine
    lines do: [:each |
        (each isSameAs: aLine)
            ifTrue: [^true]].
    ^false
!
   
isNearPoint: aPoint
    ^(self boundingBox expandBy: 2) containsPoint: aPoint
!
   
isVertex
    ^true
! !

(CodeFiler organizerFor: RegionEditorVertex) reorganizeFrom: #(
('testing' includesLine: isNearPoint: isVertex )) !

!CodeFiler forClass: RegionEditorVertex !
   
displaySelectedWith: aPen
    | oldColor |
    oldColor := aPen backColor.
    aPen
        backColor: aPen foreColor;
        place: self position; circleFilled: 4;
        backColor: oldColor
!
  
displayWith: aPen
    | box |
    box := (self position extentFromLeftTop: 1 @ 1) expandBy: 1.
    aPen fill: box color: aPen foreColor
! !

(CodeFiler organizerFor: RegionEditorVertex) reorganizeFrom: #(
('displaying' displaySelectedWith: displayWith: )) !

!CodeFiler forClass: RegionEditorVertex !
   
printInternalOn: aStream
    position printOn: aStream
! !

(CodeFiler organizerFor: RegionEditorVertex) reorganizeFrom: #(
('printing' printInternalOn: )) !

!CodeFiler forClass: RegionEditorVertex !
 
from: aPoint
    position := aPoint
! !

(CodeFiler organizerFor: RegionEditorVertex) reorganizeFrom: #(
('private' from: )) !
CodeFiler setCommentFor: RegionEditorVertex to: ''!


!CodeFiler forClass: ScrollingPane class ! !

(CodeFiler organizerFor: ScrollingPane class) reorganizeFrom: #() !

!CodeFiler forClass: ScrollingPane class !
 
constructEventsTriggered
    "Private - answer the set of events that instances of the receiver can trigger."

    "ScrollingPane initializeEventsTriggered"

    ^super constructEventsTriggered
        addAll: #(
            display:
            button1Down:
            button1Up:
            button2Down:
            button2Up:
            button1DoubleClicked:
            button2DoubleClicked:
            button1Moved:
            button2Moved:
            mouseMoved:);
        yourself



! !

(CodeFiler organizerFor: ScrollingPane class) reorganizeFrom: #(
('supported events' constructEventsTriggered )) !


!CodeFiler forClass: ScrollingPane !
   
scrollPosition: aPoint
    scrollPosition := aPoint
!
 
lastModifierKeys: anInteger
    lastModifierKeys := anInteger
!
   
lastModifierKeys
    ^lastModifierKeys
!
  
visibleRectangle
    "This is the visible area of the canvas."
    ^self scrollPosition extent: self extent

!
  
mouseLocation: aPoint
    mouseLocation := aPoint.
    self isActive ifTrue: [self setFocus].
	 aPoint == nil ifTrue: [ self halt ]
!
   
mouseLocation
    ^mouseLocation
!

scrollPosition
    ^scrollPosition
! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('accessing' scrollPosition: lastModifierKeys: lastModifierKeys visibleRectangle mouseLocation: mouseLocation scrollPosition )) !

!CodeFiler forClass: ScrollingPane !
   
pen
    "Answer my pen.  Some Digitalk thing resets the viewport org
     periodically.  Set it to the correct value."
    ^super pen setViewportOrg: self scrollPosition negated
! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('graphics tool' pen )) !

!CodeFiler forClass: ScrollingPane !
  
initialize
    super initialize.
    self
        initializeLastModifierKeys;
        initializeScrollRectangle;
        initializeScrollPosition!
 
initializeLastModifierKeys
    "Set my lastModifierKeys to no keys."
    self lastModifierKeys: 0!

initializeScrollPosition
    "Set my scroll position to the minimum scroll position."
    self scrollPosition: self minimumScroll!

initializeScrollRectangle
    "Set my scroll rectangle to a rectangle the size of display."
    self privateScrollRectangle: Display boundingBox
! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('initializing' initialize initializeLastModifierKeys initializeScrollPosition initializeScrollRectangle )) !

!CodeFiler forClass: ScrollingPane !

invalidateRect: aRectangle
    "When I invalidate myself, invalidate in canvas coordinates."
    ^aRectangle isNil
        ifTrue: [super invalidateRect: aRectangle erase: false]
        ifFalse: [super invalidateRect: (self mapCanvasToPane: aRectangle) erase: false]
!
  
invalidate
	"When I invalidate myself, invalidate in canvas coordinates."
	self invalidateRect: self visibleRectangle!

update
	super update.
	self invalidate! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('invalidating' invalidateRect: invalidate update )) !

!CodeFiler forClass: ScrollingPane !
   
isShiftKeyDown
    "Answer true if the shift key is down."
    ^(self lastModifierKeys bitAnd: MkShift) ~=0!
  
isControlKeyDown
    "Answer true if the control key is down."
    ^(self lastModifierKeys bitAnd: MkControl) ~=0! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('modifier keys' isShiftKeyDown isControlKeyDown )) !

!CodeFiler forClass: ScrollingPane !
 
scrollToMakeRectangleVisible: aRectangle
    "Scroll myself in order to make aRectangle visible."
    self scrollTopCorner:
        (self horizontalScrollToMakeRectangleVisible: aRectangle) @
            (self verticalScrollToMakeRectangleVisible: aRectangle)!

scrollRectangle: aRectangle
    "Set the rectangle that I can scroll over to aRectangle."
    | oldScrollRectangle newScrollPosition |
    oldScrollRectangle := self scrollRectangle.
    newScrollPosition := aRectangle leftTop
        rightAndDown:
            aRectangle extent * (self scrollPosition - oldScrollRectangle leftTop) abs // oldScrollRectangle height.
    self
        privateScrollRectangle: aRectangle;
        scrollPosition: newScrollPosition;
        setScrollRanges;
        updateSliders
!
  
scrollRectangle
    ^scrollRectangle
!

scrollExtent
    "Answer the extent of my scrollable area."
    ^self scrollRectangle extent! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('scrolling' scrollToMakeRectangleVisible: scrollRectangle: scrollRectangle scrollExtent )) !

!CodeFiler forClass: ScrollingPane !
  
whileButton1DownDo: block
    "Evaluate block with the point at which each button1Move event
     occurs until the next button1Up event."
    | aPoint |
    self captureMouseInput.
    Notifier consumeInputUntil: [:event |
        event selector == #button1Move:
            ifTrue: [
        			aPoint := event arguments first.
        			self mouseLocation: aPoint.
					block value: aPoint]
        		ifFalse: [
					event selector == #button1Up:
            		ifTrue: [
        					aPoint := event arguments first.
        					self mouseLocation: aPoint]].
        event selector == #button1Up:].
    self clearMouseCapture
! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('polling' whileButton1DownDo: )) !

!CodeFiler forClass: ScrollingPane !
  
mapCanvasToPane: aPointOrRectangle
    ^aPointOrRectangle translateBy: self scrollPosition negated
!
  
mapPaneToCanvas: aPointOrRectangle
    ^aPointOrRectangle translateBy: self scrollPosition
! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('coordinate transforms' mapCanvasToPane: mapPaneToCanvas: )) !

!CodeFiler forClass: ScrollingPane !
 
privateScrollRectangle: aRectangle
    scrollRectangle := aRectangle
! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('private-accessing' privateScrollRectangle: )) !

!CodeFiler forClass: ScrollingPane !
 
displayIn: aRectangle
    "Private - Display the area inside aRectangle. Triggered by
    wmPaint:with: caused by message invalidate."
    self triggerEvent: #display: with: aRectangle.
    self triggerEvent: #display
!

erase
	self pen fill: self visibleRectangle color: self backColor! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('private-displaying' displayIn: erase )) !

!CodeFiler forClass: ScrollingPane !

horizontalScrollToMakeRectangleVisible: aRectangle
    "Private - Answer the number of pixels to scroll horizontally to make aRectangle
     visible."
    | rectangleLeft rectangleRight visibleLeft visibleRight |
    rectangleLeft := aRectangle left.
    rectangleRight := aRectangle right.
    visibleLeft := self scrollPosition x.
    visibleRight := visibleLeft right: self rectangle width.
    rectangleLeft < visibleLeft ifTrue: [^visibleLeft - rectangleLeft].
    rectangleRight > visibleRight ifTrue: [^visibleRight - rectangleRight].
    ^0
!
   
verticalScrollToMakeRectangleVisible: aRectangle
    "Private - Answer the number of pixels to scroll vertically to make aRectangle
     visible."
    | rectangleTop rectangleBottom visibleTop visibleBottom |
    rectangleTop := aRectangle top.
    rectangleBottom := aRectangle bottom.
    visibleTop := self scrollPosition y.
    visibleBottom := visibleTop down: self rectangle height.
    rectangleTop < visibleTop ifTrue: [^visibleTop - rectangleTop].
    rectangleBottom > visibleBottom ifTrue: [^visibleBottom - rectangleBottom].
    ^0
!

adjustScrollDelta: aPoint
    "Private - I am trying to scroll by aPoint.  Adjust aPoint so that it will not cause
     me to scroll outside my ranges."
    ^self scrollPosition - ((self scrollPosition - aPoint max: self minimumScroll) min: self maximumScroll)! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('private-scroll bar adjusting' horizontalScrollToMakeRectangleVisible: verticalScrollToMakeRectangleVisible: adjustScrollDelta: )) !

!CodeFiler forClass: ScrollingPane !
 
maximumScroll
    "Private - Answer the maximum value permitted for the scroll bars."
    ^self maximumHorizontalScroll @ self maximumVerticalScroll!
 
minimumHorizontalScroll
    "Private - Answer the minimum horizontal scroll position."
    ^self scrollRectangle left!

minimumVerticalScroll
    "Private - Answer the minimum vertical scroll position."
    ^self scrollRectangle top!
 
minimumScroll
    "Private - Answer the minimum value permitted for the scroll bars."
    ^self minimumHorizontalScroll @ self minimumVerticalScroll!
 
amountToPageLeft
    "Private - Answer the number of pixels to scroll when I page left."
    ^self rectangle width - self amountToScrollLeft!
 
amountToPageUp
    "Private - Answer the number of pixels to scroll when I page up."
    ^self rectangle height - self amountToScrollUp!
  
maximumHorizontalScroll
    "Private - Answer the maximum horizontal scroll position."
    ^self scrollRectangle right!
   
maximumVerticalScroll
    "Private - Answer the maximum vertical scroll position."
    ^self scrollRectangle bottom! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('private-scroll bar ranges' maximumScroll minimumHorizontalScroll minimumVerticalScroll minimumScroll amountToPageLeft amountToPageUp maximumHorizontalScroll maximumVerticalScroll )) !

!CodeFiler forClass: ScrollingPane !
   
updateSliders
    "Private - Update the slider boxes in the scrollbars."
    self handle = NullHandle
        ifFalse: [self
            updateVerticalSlider;
            updateHorizontalSlider;
            resizeChildren]
!
 
resizeChildren
    "Private - Resize the children of the window."
    | box |
    box := self scrollPosition negated extentFromLeftTop: self scrollExtent.
    self children notNil
        ifTrue: [self children do: [ :subpane | subpane resize: box]].
    ^nil!
  
updateHorizontalSlider
    "Private - Update the horiztonal slider position."
    UserLibrary
        setScrollPos: self asParameter
        bar: SbHorz
        position: self scrollPosition x
        redraw: true!

updateVerticalSlider
    "Private - Update the vertical slider position."
    UserLibrary
        setScrollPos: self asParameter
        bar: SbVert
        position: self scrollPosition y
        redraw: true! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('private-scroll bar updating' updateSliders resizeChildren updateHorizontalSlider updateVerticalSlider )) !

!CodeFiler forClass: ScrollingPane !
  
scrollTopCorner: aPoint
    "Private - Move my content region by aPoint.  I don't bother offsetting my
     pen (the graphics tool code just resets it anyway).  The pen method will
     set the offset to the appropriate value."
    | scrollAmount scrollX scrollY |
    scrollAmount := self adjustScrollDelta: aPoint.
    scrollAmount = (0@0) ifTrue: [^self].
    self scrollPosition: self scrollPosition - scrollAmount.
    scrollX := scrollAmount x.
    scrollY := scrollAmount y.
    UserLibrary
        scrollWindow: handle
        xAmount: scrollX
        yAmount: scrollY
        lpRect: nil
        clipRect: nil.
    self updateSliders.
    UserLibrary updateWindow: self handle
!
   
setHorizontalScrollRange
    "Private - Set the ranges for the horizontal scroll bar."
    | min max |
    (self style bitAnd: WsHscroll) = 0
        ifTrue: [min := max := 0]
        ifFalse: [
            min := self minimumHorizontalScroll.
            max := self maximumHorizontalScroll].
    UserLibrary
        setScrollRange: self asParameter
        bar: SbHorz
        min: min
        max: max
        redraw: false
!
  
setVerticalScrollRange
    "Private - Set the ranges for the vertical scroll bar."
    | min max |
    (self style bitAnd: WsVscroll) = 0
        ifTrue: [min := max := 0]
        ifFalse: [
            min := self minimumVerticalScroll.
            max := self maximumVerticalScroll].
    UserLibrary
        setScrollRange: self asParameter
        bar: SbVert
        min: min
        max: max
        redraw: false
!
  
scrollVertical: anInteger
    "Private - Scroll the text anInteger number of bits down
     (if anInteger is positive) or up (if negative)."
    self scrollTopCorner: 0 @ anInteger!

scrollHorizontal: anInteger
    "Private - Scroll the text anInteger number of bits left
     (if anInteger is positive) or right (if negative)."
    self scrollTopCorner: anInteger @ 0!
   
scrollTopCornerTo: aPoint
    "Private - Scroll by content region to aPoint."
    self scrollTopCorner: self topCorner - aPoint!
  
setScrollRanges
    "Private - Set my scroll ranges."
    self handle = NullHandle
        ifFalse: [self
            setHorizontalScrollRange;
            setVerticalScrollRange;
            updateSliders]
! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('private-scroll bars' scrollTopCorner: setHorizontalScrollRange setVerticalScrollRange scrollVertical: scrollHorizontal: scrollTopCornerTo: setScrollRanges )) !

!CodeFiler forClass: ScrollingPane !
  
validate
    "Private - Validate myself.  Set my scroll ranges."
    super validate.
    self setScrollRanges! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('private-validating' validate )) !

!CodeFiler forClass: ScrollingPane !
  
wmLbuttonup: wParam with: lParam
    "Private - Process the left button up message."
    self
        lastModifierKeys: wParam;
        sendInputEvent: #button1Up:
        with: (self mapPaneToCanvas: (WinPoint fromInteger: lParam) asPoint).
    ^nil
!
 
wmRbuttonup: wParam with: lParam
    "Private - Process the right button up message."
    self
        lastModifierKeys: wParam;
        sendInputEvent: #button2Up:
        with: (self mapPaneToCanvas: (WinPoint fromInteger: lParam) asPoint).
    ^nil
!

wmPaint: wordInteger with: longInteger
    "Private - Process the paint message."
    | displayContext paintStructure oldDC |
    (self graphicsTool isKindOf: GraphicsTool) ifFalse: [^nil].
    "If my graphics tool has a DC, then release it.  The call to beginPaint will
     provide me with one."
    (oldDC := graphicsTool privateHandle) notNil ifTrue: [self releaseDC: oldDC].
    paintStructure := ExternalAddress allocateMemory: WinPaintStructure sizeInBytes.
    displayContext :=
        DeviceContext fromInteger:
            (UserLibrary
                beginPaint: self asParameter
                paintStructure: paintStructure asParameter).
    self graphicsTool setHandle: displayContext.
    "Make myself display.  Offset the rectangle answered by the paint structure by the scroll position."
    self displayIn: (self mapPaneToCanvas: (
        WinPaintStructure fromAddress: paintStructure) rectangle asRectangle).
    "Reset my graphics tool's handle.  The following call to endPaint will free it anyway."
    self graphicsTool handle: nil.
    UserLibrary
        endPaint: self asParameter
        paintStructure: paintStructure asParameter.
    "Get rid of the paint structure."
    paintStructure free.
    ^1
!
   
mouseMove: aPoint
    "Private - The mouse has moved with no buttons down."
    self
        mouseLocation: aPoint;
        triggerEvent: #mouseMove:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super mouseMove: (
            self mapCanvasToPane: aPoint)]
!

wmLbuttondown: wParam with: lParam
    "Private - Process the left button down message."
    self
        lastModifierKeys: wParam;
        sendInputEvent: #button1Down:
        with: (self mapPaneToCanvas: (WinPoint fromInteger: lParam) asPoint).
    ^nil
!
   
button1DoubleClick: aPoint
    "Private - Left button has been double clicked."
    self
        mouseLocation: aPoint;
        triggerEvent: #button1DoubleClicked:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super button1DoubleClick: (
            self mapCanvasToPane: aPoint)]
!

button2DoubleClick: aPoint
    "Private - Right button has been double clicked."
    self
        mouseLocation: aPoint;
        triggerEvent: #button2DoubleClicked:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super button2DoubleClick: (
            self mapCanvasToPane: aPoint)]
!
   
wmRbuttondown: wParam with: lParam
    "Private - Process the right button down message."
    self
        lastModifierKeys: wParam;
        sendInputEvent: #button2Down:
        with: (self mapPaneToCanvas: (WinPoint fromInteger: lParam) asPoint).
    ^nil
!
  
button2Move: aPoint
    "Private - The mouse has been moved with the right button down."
    self
        mouseLocation: aPoint;
        triggerEvent: #button2Move:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super button2Move: (
            self mapCanvasToPane: aPoint)]
!
   
button2Down: aPoint
    "Private - The right button has been pressed down at aPoint."
    self
        mouseLocation: aPoint;
        triggerEvent: #button2Down:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super button2Down: (
            self mapCanvasToPane: aPoint)]
!
  
wmMousemove: wParam with: lParam
    "Private - Process mouse move message."
    | point |
    self lastModifierKeys: wParam.
    point := self mapPaneToCanvas: (WinPoint fromInteger: lParam) asPoint.
    (wParam bitAnd: MkLbutton) ~= 0
         ifTrue: [self
            sendMouseMoveInputEvent: #button1Move:
            with: point.
             ^nil].
     (wParam bitAnd: MkRbutton) ~= 0
        ifTrue: [self
            sendMouseMoveInputEvent: #button2Move:
            with: point.
            ^nil].
     self
        sendMouseMoveInputEvent: #mouseMove:
        with: point.
     ^nil
!

wmNchittest: wParam with: lParam
        "Private - Process Non-client mouse hit test message.
        Returns Httransparent so that it does not recieve any
        mouse messages."
    ^nil
!
   
wmVScroll: code "wordInteger" with: longInteger
    "Private - Process a WM_VSCROLL message."
    code = SbLineup ifTrue: [^self scrollVertical: self amountToScrollUp].
    code = SbLinedown ifTrue: [^self scrollVertical: self amountToScrollUp negated].
    code = SbPageup ifTrue: [^self scrollVertical: self amountToPageUp].
    code = SbPagedown ifTrue: [^self scrollVertical: self amountToPageUp negated].
    code = SbThumbposition ifTrue: [^self scrollVertical: self scrollPosition y - (longInteger bitAnd: 65535)].
    code = SbThumbtrack ifTrue: [^self scrollVertical: self scrollPosition y - (longInteger bitAnd: 65535)].
    ^nil!

wmHScroll: code "wordInteger" with: longInteger
    "Private - Process a WM_HSCROLL message."
    code = SbLineup ifTrue: [^self scrollHorizontal: self amountToScrollLeft].
    code = SbLinedown ifTrue: [^self scrollHorizontal: self amountToScrollLeft negated].
    code = SbPageup ifTrue: [^self scrollHorizontal: self amountToPageLeft].
    code = SbPagedown ifTrue: [^self scrollHorizontal: self amountToPageLeft negated].
    code = SbThumbposition ifTrue: [^self scrollHorizontal: self scrollPosition x - (longInteger bitAnd: 65535)].
    code = SbThumbtrack ifTrue: [^self scrollHorizontal: self scrollPosition x - (longInteger bitAnd: 65535)].
    ^nil
!
  
button1Move: aPoint
    "Private - The mouse has been moved with the left button down."
    self
        mouseLocation: aPoint;
        triggerEvent: #button1Move:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super button1Move: (
            self mapCanvasToPane: aPoint)]
!

button1Down: aPoint
    "Private - The left button has been pressed down at aPoint."
    self
        mouseLocation: aPoint;
        triggerEvent: #button1Down:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super button1Down: (
            self mapCanvasToPane: aPoint)]
!
   
wmLbuttondblclk: wParam with: lParam
    "Private - Process the left button double click message."
    self
        lastModifierKeys: wParam;
        sendInputEvent: #button1DoubleClick:
        with: (self mapPaneToCanvas: (WinPoint fromInteger: lParam) asPoint).
    ^nil
!
  
button1Up: aPoint
    "Private - The left button has been released."
    self
        mouseLocation: aPoint;
        triggerEvent: #button1Up:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super button1Up: (
            self mapCanvasToPane: aPoint)]
!
   
button2Up: aPoint
    "Private - The right button has been released."
    self
        mouseLocation: aPoint;
        triggerEvent: #button2Up:
        withArguments: (Array with: aPoint)
        ifNotHandled: [super button2Up: (
            self mapCanvasToPane: aPoint)]
!
  
wmRbuttondblclk: wParam with: lParam
    "Private - Process the right button double click message."
    self
        lastModifierKeys: wParam;
        sendInputEvent: #button2DoubleClick:
        with: (self mapPaneToCanvas: (WinPoint fromInteger: lParam) asPoint).
    ^nil
! !

(CodeFiler organizerFor: ScrollingPane) reorganizeFrom: #(
('private-windows events' wmLbuttonup:with: wmRbuttonup:with: wmPaint:with: mouseMove: wmLbuttondown:with: button1DoubleClick: button2DoubleClick: wmRbuttondown:with: button2Move: button2Down: wmMousemove:with: wmNchittest:with: wmVScroll:with: wmHScroll:with: button1Move: button1Down: wmLbuttondblclk:with: button1Up: button2Up: wmRbuttondblclk:with: )) !
CodeFiler setCommentFor: ScrollingPane to: ''!


!CodeFiler forClass: DrawingPane class ! !

(CodeFiler organizerFor: DrawingPane class) reorganizeFrom: #() !

!CodeFiler forClass: DrawingPane class !
 
constructEventsTriggered
    "Private - answer the set of events that instances of the receiver can trigger."

    "DrawingPane initializeEventsTriggered"

    ^super constructEventsTriggered
        add: #controlClickAt:;
        add: #objects:near: ;
        add: #objects:movedBy:;
        add: #objectsDeleted:;
        add: #objectsDisplaySelected:unselected:;
        add: #singleClickSelected:;
        add: #doubleClickSelected:;
        add: #needsMenu:;
        yourself
! !

(CodeFiler organizerFor: DrawingPane class) reorganizeFrom: #(
('supported events' constructEventsTriggered )) !


!CodeFiler forClass: DrawingPane !

selectedObjects: aCollection
    selectedObjects := aCollection
!
 
selectedObjects
    ^selectedObjects
!

objects
    ^objects
!

pen
    ^buffer pen setViewportOrg: super pen getViewportOrg
!

objects: aCollection
    objects := aCollection
! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('accessing' selectedObjects: selectedObjects objects pen objects: )) !

!CodeFiler forClass: DrawingPane !

addObject: anObject
    ^self addObject: anObject refresh: true
!
 
addObjects: aCollection
    ^self addObjects: aCollection refresh: true
!
 
addObject: anObject refresh: aBoolean
    self objects add: anObject.
    aBoolean ifTrue: [self invalidate]
!
   
addObjects: aCollection refresh: aBoolean
    aCollection do: [:object | self objects add: object].
    aBoolean ifTrue: [self invalidate]



! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('object adding' addObject: addObjects: addObject:refresh: addObjects:refresh: )) !

!CodeFiler forClass: DrawingPane !
  
deleteObject: anObject refresh: aBoolean
    self objects remove: anObject ifAbsent: [].
    self selectedObjects remove: anObject ifAbsent: [].
    aBoolean ifTrue: [self invalidate]
!
   
deleteObject: anObject
    self deleteObject: anObject refresh: true



!
  
deleteObjects: aCollection
    self deleteObjects: aCollection refresh: true


!

deleteObjects: aCollection refresh: aBoolean
    aCollection do: [:object |
        self objects remove: object ifAbsent: [].
        self selectedObjects remove: object ifAbsent: []].
    aBoolean ifTrue: [self invalidate]



! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('object deleting' deleteObject:refresh: deleteObject: deleteObjects: deleteObjects:refresh: )) !

!CodeFiler forClass: DrawingPane !
 
selectObject: anObject
    self selectObject: anObject refresh: true



!
  
selectObjects: aCollection
    self selectObjects: aCollection refresh: true


!

selectObjects: aCollection refresh: aBoolean
    self selectedObjects: aCollection.
    aBoolean ifTrue: [self invalidate]

!
   
selectObject: anObject refresh: aBoolean
    self selectedObjects: (OrderedCollection with: anObject).
    aBoolean ifTrue: [self invalidate]
! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('object selecting' selectObject: selectObjects: selectObjects:refresh: selectObject:refresh: )) !

!CodeFiler forClass: DrawingPane !
  
initialize
    super initialize.
    self objects: OrderedCollection new.
    self selectedObjects: OrderedCollection new.
!

defaultCursor
    ^CursorManager crossHair
! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('private-initializing' initialize defaultCursor )) !

!CodeFiler forClass: DrawingPane !
   
destroyBuffer
    buffer isNil ifFalse: [buffer release]
!

createBuffer
    self destroyBuffer.
    buffer := Bitmap screenExtent: self visibleRectangle extent.
    self pen
        foreColor: self foreColor;
        backColor: self backColor
! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('private-buffer' destroyBuffer createBuffer )) !

!CodeFiler forClass: DrawingPane !
  
displayIn: aRectangle
    "Triggered by wmPaint:with: caused by message invalidate."
    super displayIn: aRectangle.
    self display
!

displayObjects
    | unselected |
    unselected := self objects reject: [:object |
        self selectedObjects includes: object].
    self triggerEvent: #objectsDisplaySelected:unselected:
        with: self selectedObjects with: unselected
!
  
displayBuffer
    super pen
        copyBitmap: buffer
        from: self visibleRectangle
        to: self visibleRectangle
!
 
display
    self pen
        fill: self visibleRectangle
        color: self backColor.
    self
        displayObjects;
        displayBuffer
! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('private-displaying' displayIn: displayObjects displayBuffer display )) !

!CodeFiler forClass: DrawingPane !
 
validate
    super validate.
    self createBuffer
! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('private-validating' validate )) !

!CodeFiler forClass: DrawingPane !

sizeChanged: anExtent
    super sizeChanged: anExtent.
    self createBuffer

! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('private-resizing' sizeChanged: )) !

!CodeFiler forClass: DrawingPane !
  
moveFrom: aPoint
    "Inform owner of each move."
    | start |
    start := aPoint. self invalidate.
    self whileButton1DownDo: [:location |
        self triggerEvent: #objects:movedBy: with: self selectedObjects with: location - start.
        start := self mouseLocation.
        self display].
!
   
normalSelect: aPoint
    "Selecting a preselected object begins a move; otherwise, a new
    list of selections is computed. Inform owner afterwards."
    | list |
    list := self triggerEvent: #objects:near: with: self selectedObjects with: aPoint.
    list isEmpty ifFalse: [^self moveFrom: aPoint].
    list := self triggerEvent: #objects:near: with: self objects with: aPoint.
    self selectObjects: list.
    self triggerEvent: #singleClickSelected: with: list.
    self invalidate.
    self moveFrom: aPoint
!

controlSelect: aPoint
    "User-defined activity required. Informs owner."
    self triggerEvent: #controlClickAt: with: aPoint.
    self invalidate


!
  
shiftSelect: aPoint
    "Automatically adds/removes from selected objects. Informs owner afterwards."
    | list |
    list := self triggerEvent: #objects:near: with: self objects with: aPoint.
    list do: [:object |
		(self selectedObjects includes: object)
			ifTrue: [self selectedObjects remove: object]
			ifFalse: [self selectedObjects add: object]].
    self triggerEvent: #singleClickSelected: with: self selectedObjects.
    self invalidate

! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('private-selecting' moveFrom: normalSelect: controlSelect: shiftSelect: )) !

!CodeFiler forClass: DrawingPane !
 
close
    super close.
    buffer release


! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('private-closing' close )) !

!CodeFiler forClass: DrawingPane !
   
button1Down: aPoint
    "Handle 3 modifier possibilities in canvas coordinates."
    super button1Down: aPoint.
    self isShiftKeyDown ifTrue: [^self shiftSelect: aPoint].
    self isControlKeyDown ifTrue: [^self controlSelect: aPoint].
    ^self normalSelect: aPoint

!
  
keyboardInput: aKeyboardInputEvent
    "Private - keyboard input was received.  Process backspace, tab, carriage return."
    | virtualKey |
    ( virtualKey := aKeyboardInputEvent virtualKey ) notNil ifTrue: [
        (virtualKey == BackspaceKey or: [virtualKey == DeleteKey])
            ifTrue: [self deleteSelectedObjects]].

!
  
button2Up: aPoint
    "In canvas coordinates."
    super button2Up: aPoint.
    self triggerEvent: #needsMenu: with: self selectedObjects
!
 
deleteSelectedObjects
    | candidates |
    candidates := self selectedObjects shallowCopy.
    self deleteObjects: candidates refresh: false.
    self triggerEvent: #objectsDeleted: with: candidates; invalidate
!
 
button1DoubleClick: aPoint
    "In canvas coordinates."
    super button1DoubleClick: aPoint.
    self triggerEvent: #doubleClickSelected: with: self selectedObjects.
    self invalidate
! !

(CodeFiler organizerFor: DrawingPane) reorganizeFrom: #(
('private-window events' button1Down: keyboardInput: button2Up: deleteSelectedObjects button1DoubleClick: )) !
CodeFiler setCommentFor: DrawingPane to: ''!
